/****************************************************************************
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *~                             IR Kernel                                    ~
 *~                         Copyright(C) 1993                                ~
 *~                       Software Kinetics Ltd.                             ~
 *~                        All Rights Reserved                               ~
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright Software Kinetics Ltd.
 * 
 * Software Kinetics Ltd. has granted a licence of this software to
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve,
 * reproduce and sell the software.
 *
 * The software is provided to the user "as is" without warranty of
 * any kind.  Neither Software Kinetics Ltd. nor the National Library
 * of Canada warrants,guarantees or makes any representation regarding
 * the use, or the results of the use of the software in terms of
 * accuracy,reliability, or otherwise.  The entire risk as results and
 * performance of the software is assumed by the user. Neither
 * Software Kinetics Ltd. nor the National Library of Canada shall be
 * liable for any direct, indirect, consequential or incidental
 * damages arising out the use of software.
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 *
 * Program: irerror.c
 *
 * Purpose: This file contains the error handling functions.
 *
 * Invocation:
 *
 * Input Arguments:
 *
 * Output/Exit Values: None
 *
 * Functions used:
 *                 display_error()
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: IR KERNEL - TARGET PRELIMINARY SOFTWARE DESIGN DOCUMENT
 *            IR KERNEL - TARGET SOFTWARE DETAIL DESIGN DOCUMENT
 *            IR KERNEL - INTERFACE REQUIREMENTS SPECIFICATION
 *            OSI - ANSI Z39.50 DOCUMENTATION
 *
 * Notes:
 *
 * $Id: irerror.c,v 1.1 1993/11/01 17:03:51 irsuprt Exp irsuprt $
 * $Log: irerror.c,v $
 * Revision 1.1  1993/11/01  17:03:51  irsuprt
 * Initial revision
 *
 *
 *****************************************************************************/

#include <stdio.h>
#include <time.h>
#include <unistd.h>
#include "ir.h"
#include "irerror.h"
#include "irconst.h"

#define ERROR_RECORD_LENGTH 135

char error_message[MAX_ERROR_MESSAGES][73] =
{
     /* 00 */ "NO ERROR",
     /* 01 */ "PROTOCOL ERROR",
     /* 02 */ "NOT ENOUGH MEMORY",
     /* 03 */ "CANNOT CREATE SOCKET",
     /* 04 */ "CANNOT BIND SOCKET",
     /* 05 */ "CANNOT ACCEPT CONNECTION",
     /* 06 */ "TIMEOUT DUE TO NO RESPONSE TO A REQUEST",
     /* 07 */ "CANNOT CONVERT BER RECORD",
     /* 08 */ "CANNOT CREATE TCP HANDLER",
     /* 09 */ "RPC CALL ERROR",
     /* 10 */ "UNABLE TO ALLOCATE THE REQUIRED MEMORY RESOURCES",
     /* 11 */ "UNABLE TO INVOKE THE INIT PROCESS",
     /* 12 */ "UNABLE TO INVOKE THE TERMINATE PROCESS",
     /* 13 */ "UNABLE TO COMMUNICATE WITH THE INIT PROCESS",
     /* 14 */ "UNABLE TO COMMUNICATE WITH THE QRM PROCESS",
     /* 15 */ "AN IR RESPONSE WAS EXPECTED FROM THE ORIGIN",
     /* 16 */ "AN IR RESPONSE WAS EXPECTED FROM THE TARGET PROCESS",
     /* 17 */ "UNABLE TO COMMUNICATE WITH THE ORIGIN SYSTEM",
     /* 18 */ "IR-ABORT REQUESTED BY A TARGET PROCESS",
     /* 19 */ "CANNOT CONNECT DUE TO DUPLICATE CLIENT",
     /* 20 */ "CANNOT GET HOST NAME",
     /* 21 */ "CLIENT NOT IN LIST",
     /* 22 */ "CANNOT OPEN FILE FOR WRITING",
     /* 23 */ "INTERNAL SYSTEM ERROR",
     /* 24 */ "CANNOT GET SOCKET NAME",
     /* 25 */ "CANNOT SEND DATA TO IR KERNEL FROM TARGET PROCESS",
     /* 26 */ "CANNOT DECODE APDU",
     /* 27 */ "CANNOT FIND THE LENGTH OF APDU",
     /* 28 */ "CANNOT SET SOCKET OPTION",
     /* 29 */ "FAILED ON LISTEN ON SOCKET",
     /* 30 */ "CANNOT DO A SELECT ON TCP PORT",
     /* 31 */ "APDU TYPE NOT ACCEPTED BY QRM",
     /* 32 */ "QUERY TYPE NOT SUPPORTED BY TARGET PROCESS",
     /* 33 */ "INVALID INIT REQUEST OPTIONS",
     /* 34 */ "INVALID INIT REQUEST MESSAGE SIZE",
     /* 35 */ "ACCESS CONTROL FAILED",
     /* 36 */ "RESOURCE CONTROL FAILED",
     /* 37 */ "ABORT BECAUSE IT IDLES TOO LONG",
     /* 38 */ "INCOMPLETE DEFAULT PARAMETERS FILE",
     /* 39 */ "INVALID INIT PROTOCOL VERSION",
     /* 40 */ "EMPTY INIT PROTOCOL VERSION",
     /* 41 */ "EMPTY INIT PREFERRED MESSAGE SIZE",
     /* 42 */ "EMPTY INIT MAXIMUM RECORD SIZE",
     /* 43 */ "SEARCH SMALL SET VALUE IS INVALID",
     /* 44 */ "SEARCH SMALL SET VALUE IS TOO LARGE",
     /* 45 */ "SEARCH MEDIUM SET VALUE IS TOO LARGE",
     /* 46 */ "SEARCH LARGE SET VALUE IS INVALID",
     /* 47 */ "SEARCH REPLACE INDICATOR VALUE IS INVALID",
     /* 48 */ "SEARCH RESULT SET NAME IS EMPTY",
     /* 49 */ "SEARCH DATABASE NAME IS EMPTY",
     /* 50 */ "SEARCH QUERY TYPE IS INVALID",
     /* 51 */ "SEARCH QUERY IS EMPTY",
     /* 52 */ "PRESENT RESULT SET ID IS EMPTY",
     /* 53 */ "PRESENT RESULT SET START POSITION IS ZERO",
     /* 54 */ "PRESENT NUMBER OF RECORDS REQUESTED IS ZERO",
     /* 55 */ "DELETE OPERATION IS INVALID",
     /* 56 */ "INVALID TRIGGER RESOURCE OPERATION",
     /* 57 */ "CANNOT GET DOMAIN NAME"
};

int fatal_error[MAX_ERROR_MESSAGES] =
{
    NO,                                /* "NO ERROR" */
    YES,                        /* "PROTOCOL ERROR" */
    YES,                        /* "NOT ENOUGH MEMORY" */
    YES,                        /* "CANNOT CREATE SOCKET" */
    YES,                        /* "CANNOT BIND SOCKET" */
    YES,                        /* "CANNOT ACCEPT CONNECTION" */
    YES,                        /* "TIMEOUT FROM READING COMM PORT" */
    YES,                        /* "CANNOT CONVERT BER RECORD" */
    YES,                        /* "CANNOT CREATE TCP HANDLER" */
    YES,                        /* "RPC CALL ERROR" */
    YES,                        /* "UNABLE TO ALLOCATE THE REQUIRED MEMORY
                                 * RESOURCES" */
    YES,                        /* "UNABLE TO INVOKE THE INIT PROCESS" */
    YES,                        /* "UNABLE TO INVOKE THE TERMINATE PROCESS" */
    YES,                        /* "UNABLE TO COMMUNICATE WITH THE INIT
                                 * PROCESS" */
    YES,                        /* "UNABLE TO COMMUNICATE WITH THE QRM
                                 * PROCESS" */
    YES,                        /* "AN IR RESPONSE WAS EXPECTED FROM THE
                                 * ORIGIN" */
    YES,                        /* "AN IR RESPONSE WAS EXPECTED FROM THE
                                 * TARGET PROCESS" */
    YES,                        /* "UNABLE TO COMMUNICATE WITH THE ORIGIN
                                 * SYSTEM" */
    YES,                        /* "IR-ABORT REQUESTED BY A TARGET PROCESS" */
    YES,                        /* "CANNOT CONNECT DUE TO DUPLICATE CLIENT" */
    YES,                        /* "CANNOT GET HOST NAME" */
    YES,                        /* "CLIENT NOT IN LIST" */
    YES,                        /* "CANNOT OPEN FILE FOR WRITING" */
    YES,                        /* "INTERNAL SYSTEM ERROR" */
    YES,                        /* "CANNOT GET SOCKET NAME" */
    YES,                        /* "CANNOT SEND DATA TO IR KERNEL FROM TARGET
                                 * PROCESS" */
    YES,                        /* "CANNOT DECODE APDU" */
    YES,                        /* "CANNOT FIND THE LENGTH OF APDU" */
    YES,                        /* "CANNOT SET SOCKET OPTION" */
    YES,                        /* "FAILED ON LISTEN ON SOCKET" */
    YES,                        /* "CANNOT DO A SELECT ON TCP PORT" */
    YES,                        /* "APDU TYPE NOT ACCEPTED BY QRM" */
    YES,                        /* "QUERY TYPE NOT SUPPORTED BY TARGET
                                 * PROCESS" */
    YES,                        /* "INVALID INIT REQUEST OPTIONS", */
    YES,                        /* "INVALID INIT REQUEST MESSAGE SIZE" */
    YES,                        /* "ACCESS CONTROL FAILED" */
    YES,                        /* "RESOURCE CONTROL FAILED" */
    YES,                        /* "ABORT BECAUSE IT IDLES TOO LONG" */
    YES,                        /* "INCOMPLETE DEFAULT PARAMETERS FILE" */
    YES,                        /* "INVALID INIT PROTOCOL VERSION" */
    YES,                        /* "EMPTY INIT PROTOCOL VERSION" */
    YES,                        /* "EMPTY INIT PREFERRED MESSAGE SIZE" */
    YES,                        /* "EMPTY INIT MAXIMUM RECORD SIZE" */
    YES,                        /* "SEARCH SMALL SET VALUE IS INVALID" */
    YES,                        /* "SEARCH SMALL SET VALUE IS TOO LARGE" */
    YES,                        /* "SEARCH MEDIUM SET VALUE IS TOO LARGE" */
    YES,                        /* "SEARCH LARGE SET VALUE IS INVALID" */
    YES,                        /* "SEARCH REPLACE INDICATOR VALUE IS
                                 * INVALID" */
    YES,                        /* "SEARCH RESULT SET NAME IS EMPTY" */
    YES,                        /* "SEARCH DATABASE NAME IS EMPTY" */
    YES,                        /* "SEARCH QUERY TYPE IS INVALID" */
    YES,                        /* "SEARCH QUERY IS EMPTY" */
    YES,                        /* "PRESENT RESULT SET ID IS EMPTY" */
    YES,                        /* "PRESENT RESULT SET START POSITION IS
                                 * ZERO" */
    YES,                        /* "PRESENT NUMBER OF RECORDS REQUESTED IS
                                 * ZERO" */
    YES,                        /* "DELETE OPERATION IS INVALID" */
    YES,                        /* "INVALID TRIGGER RESOURCE OPERATION" */
	YES 						/* CANNOT GET DOMAIN NAME */
};

char ERR_STR[ERROR_RECORD_LENGTH];

FILE *ERROR_FILE;


/*****************************************************************************
 *
 * Function: write_error_file
 *
 * Purpose: This function writes the error message to the error file
 *          irtarget.err and it appends spaces and a carriage return to
 *                   make up a fixed length error record.
 *
 * Input Arguments:
 *              errfile - error file handler
 *                                str - error message string
 *                                error_record_length - maximum record length
 *
 * External Variables:
 *
 * Output Return values:
 *
 * Output Arguments:
 *
 * External Variables:
 *
 * Fucntion used:
 *
 * Algorithm:
 *
 * Version: Date:   93/06/24
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: IR KERNEL - TARGET PRELIMINARY SOFTWARE DESIGN DOCUMENT
 *            IR KERNEL - TARGET SOFTWARE DETAIL DESIGN DOCUMENT
 *
 * Notes:
 *
 ****************************************************************************/
write_error_file(errfile, str, error_record_length)
    FILE *errfile;
    char *str;
    int error_record_length;
{

    int i;

    fprintf(errfile, "%s", str);
    for (i = strlen(str); i < (error_record_length - 1); i++)
        fprintf(errfile, " ");
    fprintf(errfile, "\n");

}

/*****************************************************************************
 *
 * Function: update_error_file
 *
 * Purpose: This function updates the error file irtarget.err and it will
 *          wrap around when it reaches to the maximum number of allowed
 *          records. The parameter is set in the parameter file irtarget.def.
 *
 * Input Arguments: message - error message string
 *
 * External Variables:
 *
 * Output Return values:
 *
 * Output Arguments:
 *
 * External Variables:
 *
 * Fucntion used:
 *
 * Algorithm:
 *
 * Version: Date:   93/06/24
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: IR KERNEL - TARGET PRELIMINARY SOFTWARE DESIGN DOCUMENT
 *            IR KERNEL - TARGET SOFTWARE DETAIL DESIGN DOCUMENT
 *
 * Notes:
 *
 ****************************************************************************/
update_error_file(message)
    char *message;
{

    long last_record;
    FILE *errfile;
    char str[ERROR_RECORD_LENGTH];

    if ((errfile = fopen(ERROR_FILE_NAME, "r+")) == NULL)
    {
        fclose(errfile);
        errfile = fopen(ERROR_FILE_NAME, "w");
        sprintf(str, "1");
        write_error_file(errfile, str, ERROR_RECORD_LENGTH);
        fclose(errfile);
        errfile = fopen(ERROR_FILE_NAME, "r+");
    }

    fscanf(errfile, "%ld", &last_record);

    if (last_record >= MAX_ERROR_RECORDS)
    {
        last_record = 1;
        fseek(errfile, (last_record * ERROR_RECORD_LENGTH), SEEK_SET);
    }
    else
    {
        fseek(errfile, (last_record * ERROR_RECORD_LENGTH), SEEK_SET);
    }
    last_record += 1;
    write_error_file(errfile, message, ERROR_RECORD_LENGTH);
    fseek(errfile, 0, SEEK_SET);
    sprintf(str, "%ld", last_record);
    write_error_file(errfile, str, ERROR_RECORD_LENGTH);

    fclose(errfile);

}

/*****************************************************************************
 *
 * Function: display_error()
 *
 * Purpose: This function displays the error message and the extra error code.
 *          If it's a non-fatal error, it will write it to the error log file.
 *
 * Input Arguments: error_code - an error code defined in irerror.h
 *                  extra_error_code - an extra user defined error code
 *
 * External Variables:
 *
 * Output Return values:
 *
 * Argument:
 *
 * External Variables:
 *
 * Function used:
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * References:
 *
 * Notes:
 *
 ****************************************************************************/
display_error(char msg[], int error_code, int extra_error_code)
{
    time_t now;
    struct tm *today_time;
    char str[ERROR_RECORD_LENGTH];
    long last_record;

#if (defined(DEBUG_LEVEL1) || defined(DEBUG_LEVEL2) || defined(DEBUG_LEVEL3))
    if (error_code == DISPLAY_USER_ERROR)
        printf("ERROR (%s): %s (%d, %d)\n", msg, ERR_STR, error_code, extra_error_code);
    else
        printf("ERROR (%s): %s (%d, %d)\n", msg, error_message[error_code], error_code, extra_error_code);
#endif

    /* Write the error message out to a disk file. */
    /* This error file is being updated in a real time fashion. */

    now = time(NULL);
    today_time = localtime(&now);

    if (error_code == DISPLAY_USER_ERROR)
        sprintf(str, "USER-DEFINED ERROR: %d/%d/%d %d:%d:%d %s, %s(%d)",
                today_time->tm_mon, today_time->tm_mday, today_time->tm_year,
                today_time->tm_hour, today_time->tm_min, today_time->tm_sec,
                msg, ERR_STR, extra_error_code);
    else
    {
        if (fatal_error[error_code] == NO)
            sprintf(str, "NON-FATAL ERROR: %2d/%2d/%2d %2d:%2d:%2d %s, %s(%3d)",
               today_time->tm_mon, today_time->tm_mday, today_time->tm_year,
                today_time->tm_hour, today_time->tm_min, today_time->tm_sec,
                    msg, error_message[error_code], extra_error_code);
        else
            sprintf(str, "FATAL ERROR: %2d/%2d/%2d %2d:%2d:%2d %s, %s(%3d)",
               today_time->tm_mon, today_time->tm_mday, today_time->tm_year,
                today_time->tm_hour, today_time->tm_min, today_time->tm_sec,
                    msg, error_message[error_code], extra_error_code);
    }

    update_error_file(str);

    return (SUCCESS);
}
