/****************************************************************************
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *~                             IR Kernel                                    ~
 *~                         Copyright(C) 1993                                ~
 *~                       Software Kinetics Ltd.                             ~
 *~                        All Rights Reserved                               ~
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright Software Kinetics Ltd.
 * 
 * Software Kinetics Ltd. has granted a licence of this software to
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve,
 * reproduce and sell the software.
 *
 * The software is provided to the user "as is" without warranty of
 * any kind.  Neither Software Kinetics Ltd. nor the National Library
 * of Canada warrants,guarantees or makes any representation regarding
 * the use, or the results of the use of the software in terms of
 * accuracy,reliability, or otherwise.  The entire risk as results and
 * performance of the software is assumed by the user. Neither
 * Software Kinetics Ltd. nor the National Library of Canada shall be
 * liable for any direct, indirect, consequential or incidental
 * damages arising out the use of software.
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 *
 * Program: irfwrite.c
 *
 * Purpose: This function is the replacement of the C function fwrite().
 *          It displays the Hex numeric value of the non-printable characters
 *          instead of the real characters that might mess up the screen since
 *            some are treated as control characters.
 *
 * Invocation:
 *
 * Input Arguments:
 *
 * Output/Exit Values: None
 *
 * Functions used:
 *                 ir_fwrite()
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: IR KERNEL - TARGET PRELIMINARY SOFTWARE DESIGN DOCUMENT
 *            IR KERNEL - TARGET SOFTWARE DETAIL DESIGN DOCUMENT
 *            IR KERNEL - INTERFACE REQUIREMENTS SPECIFICATION
 *            OSI - ANSI Z39.50 DOCUMENTATION
 *
 * Notes:
 *
 * $Id: irfwrite.c,v 1.1 1993/11/01 17:03:51 irsuprt Exp irsuprt $
 * $Log: irfwrite.c,v $
 * Revision 1.1  1993/11/01  17:03:51  irsuprt
 * Initial revision
 *
 *
 *****************************************************************************/

#include <stdio.h>
#include <string.h>

/*****************************************************************************
 *
 * Function: ir_fwrite
 *
 * Purpose: This function is the replacement of the C function fwrite().
 *          It displays the Hex numeric value of the non-printable characters
 *          instead of the real characters that might mess up the screen since
 *          some are treated as control characters.
 *
 * Input Arguments:
 *          str - character string pointer
 *          len - string str length
 *          repeat - ignored
 *
 * Return values: none
 *
 * Output Arguments:
 *          outfile - File handler to be used to output the content of the
 *                    string str.
 *
 * External Variables:
 *
 * Function used:
 *
 * Algorithm: Based on ISO ANSI Z39.50 IR kernel state machine.
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: IR KERNEL - TARGET PRELIMINARY SOFTWARE DESIGN DOCUMENT
 *            IR KERNEL - TARGET SOFTWARE DETAIL DESIGN DOCUMENT
 *            IR KERNEL - INTERFACE REQUIREMENTS SPECIFICATION
 *            OSI - ANSI Z39.50 DOCUMENTATION
 *
 * Notes:
 *
 ****************************************************************************/
void
ir_fwrite(str, len, repeat, outfile)
    unsigned char *str;
    int len, repeat;
    FILE *outfile;
{

    int i, n, prev_type;
    char num_str[100];

    n = 0;
    for (i = 0; i < len; i++)
    {
        if (str[i] >= 0x20 && str[i] <= 0x7e)
        {
            if (prev_type == 1 || n == 0)
            {
                fprintf(outfile, "%c", str[i]);
                n += 1;
            }
            else
            {
                fprintf(outfile, "|%c", str[i]);
                n += 2;
            }
            prev_type = 1;
        }
        else
        {
            if (prev_type == 2 || n == 0)
                sprintf(num_str, "%x,", str[i]);
            else
                sprintf(num_str, "|%x,", str[i]);
            prev_type = 2;
            n += strlen(num_str);
            fprintf(outfile, "%s", num_str);
        }

        if (n > 75)
        {
            n = 0;
            fprintf(outfile, "||\n");
        }
    }

    fprintf(outfile, "\n");
}
