/****************************************************************************
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *~                             IR Kernel                                    ~
 *~                         Copyright(C) 1993                                ~
 *~                       Software Kinetics Ltd.                             ~
 *~                        All Rights Reserved                               ~
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright Software Kinetics Ltd.
 * 
 * Software Kinetics Ltd. has granted a licence of this software to
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve,
 * reproduce and sell the software.
 *
 * The software is provided to the user "as is" without warranty of
 * any kind.  Neither Software Kinetics Ltd. nor the National Library
 * of Canada warrants,guarantees or makes any representation regarding
 * the use, or the results of the use of the software in terms of
 * accuracy,reliability, or otherwise.  The entire risk as results and
 * performance of the software is assumed by the user. Neither
 * Software Kinetics Ltd. nor the National Library of Canada shall be
 * liable for any direct, indirect, consequential or incidental
 * damages arising out the use of software.
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 *
 * Program: irinit.c
 *
 * Purpose: This file contains the IR initialize functions.
 *
 * Invocation:
 *
 * Input Arguments:
 *
 * Output/Exit Values: None
 *
 * Functions used: ir_init_request()
 *                 ir_init_response()
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: IR KERNEL - TARGET PRELIMINARY SOFTWARE DESIGN DOCUMENT
 *            IR KERNEL - TARGET SOFTWARE DETAIL DESIGN DOCUMENT
 *
 * Notes:
 *
 * $Id: irinit.c,v 1.2 1993/11/02 15:40:54 irsuprt Exp irsuprt $
 * $Log: irinit.c,v $
 * Revision 1.2  1993/11/02  15:40:54  irsuprt
 * removed ; of line 130 for ANSI compliance (PROBLEM 13)
 *
 * Revision 1.1  1993/11/01  17:03:51  irsuprt
 * Initial revision
 *
 *
 *****************************************************************************/
/* For RPC */
#include <stdio.h>
#include <rpc/rpc.h>

#include "ir.h"
#include "irstatab.h"
#include "irconst.h"
#include "irerror.h"
#include "irassoc.h"
#include "asn_incl.h"

/*****************************************************************************
 *
 * Function: ir_init_request
 *
 * Purpose: To start the init/QRM target process by using the RPC call and
 *          then transmit the init request APDU to the target process.
 *
 * Input Arguments: assocId     - current association
 *                  tp_hostname - target process host name
 *
 * External Variables:
 *
 * Return values:
 *
 * Output Argument:
 *
 * External Variables:
 *
 * Function called: cs_send_APDU()
 *
 * Version:
 *
 * References:
 *
 * Notes:
 *
 ****************************************************************************/
int
ir_init_request(assocId, tp_hostname)
    struct assoc_struct **assocId;
    char *tp_hostname;
{
    int result;

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "Calling ir_init_request ...\n");
#endif

    result = ira_invoke_target_process(tp_hostname, assocId);

#ifdef DEBUG_LEVEL1
    if (result == SUCCESS)
        fprintf(DEBUG_FILE1, "successfully invoke target process tpbbox\n");
    else
        fprintf(DEBUG_FILE1, "failed invoke target process tpbbox\n");
#endif

    if (result == SUCCESS)
    {
        result = ira_service_request(assocId);

#ifdef DEBUG_LEVEL1
        if (result == SUCCESS)
            fprintf(DEBUG_FILE1, "successfully sent init request to target process\n");
        else
            fprintf(DEBUG_FILE1, "failed sent init request to target process\n");
#endif
    }

    return (result);
}


/*****************************************************************************
 *
 * Function: ir_init_response
 *
 * Purpose: To transmit the init response APDU from the target process system
 *          to the origin system by calling the communication service function
 *          cs_send_APDU().
 *
 * Input Arguments:
 *      struct NetIO *from_target_process
 *              - The pointer to the decoded APDU data structure from
 *                the target process system.
 *      struct NetIO *to_client
 *              - The pointer to the decoded APDU data structure for
 *                sending to origin system.
 *
 * External Variables:
 *
 * Output Return values:
 *
 * Output Argument:
 *
 * External Variables:
 *
 * Function used: cs_send_APDU()
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * References:
 *
 * Notes:
 *
 ****************************************************************************/
ir_init_response(assocId, PDU_ptr)
    struct assoc_struct **assocId;
    APDU *PDU_ptr;
{

    int result;

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "Calling ir_init_response\n");
#endif

    result = ira_send_response(assocId);

#ifdef DEBUG_LEVEL1
    if (result == SUCCESS)
        fprintf(DEBUG_FILE1, "successfully sent init response\n");
    else
        fprintf(DEBUG_FILE1, "failed sent init response\n");
#endif

#ifdef QRM
    /* if QRM then establish the QRM connection else it's a black box case */
    /* establish QRM */
    if (result == SUCCESS)
    {

#ifdef DEBUG_LEVEL1
        fprintf(DEBUG_FILE1, "ir_init_response: establish QRM\n");
#endif

        close((*assocId)->socket_no[QRMSOCK]);        /* close INIT socket */
        result = cs_connect(3027, TEMP_HOST_NAME, assocId);
    }
#endif

    return (result);

}
