/****************************************************************************
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *~                             IR Kernel                                    ~
 *~                         Copyright(C) 1993                                ~
 *~                       Software Kinetics Ltd.                             ~
 *~                        All Rights Reserved                               ~
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright Software Kinetics Ltd.
 * 
 * Software Kinetics Ltd. has granted a licence of this software to
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve,
 * reproduce and sell the software.
 *
 * The software is provided to the user "as is" without warranty of
 * any kind.  Neither Software Kinetics Ltd. nor the National Library
 * of Canada warrants,guarantees or makes any representation regarding
 * the use, or the results of the use of the software in terms of
 * accuracy,reliability, or otherwise.  The entire risk as results and
 * performance of the software is assumed by the user. Neither
 * Software Kinetics Ltd. nor the National Library of Canada shall be
 * liable for any direct, indirect, consequential or incidental
 * damages arising out the use of software.
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 *
 * Program: irsearch.c
 *
 * Purpose: This file contains the search service functions.
 *
 * Invocation:
 *
 * Input Arguments:
 *
 * Output/Exit Values: None
 *
 * Functions used: ir_search_request()
 *                 ir_search_response()
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: IR KERNEL - TARGET PRELIMINARY SOFTWARE DESIGN DOCUMENT
 *            IR KERNEL - TARGET SOFTWARE DETAIL DESIGN DOCUMENT
 *            IR KERNEL - INTERFACE REQUIREMENTS SPECIFICATION
 *            OSI - ANSI Z39.50 DOCUMENTATION
 *
 * Notes:
 *
 * $Id: irsearch.c,v 1.1 1993/11/01 17:03:51 irsuprt Exp irsuprt $
 * $Log: irsearch.c,v $
 * Revision 1.1  1993/11/01  17:03:51  irsuprt
 * Initial revision
 *
 *
 *****************************************************************************/

#include <stdio.h>
#include "ir.h"
#include "irerror.h"
#include "irconst.h"
#include "asn_incl.h"

/*****************************************************************************
 *
 * Function: ir_search_request
 *
 * Purpose: This function receives a search request from the Origin,
 *          validates it, and relays the APDU to the target process.
 *
 * Input Arguments:    assocId        - pointer to the current association
 *                     PDU_ptr        - pointer to the search request PDU
 *
 * External Variables:
 *
 * Return values:      SUCCESS
 *                     FAILED
 *
 * Output Argument:
 *
 * External Variables:
 *
 * Function used:
 *                  ira_service_request()
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * References:
 *
 * Notes:
 *
 ****************************************************************************/
int
ir_search_request(assocId, PDU_ptr)
    struct assoc_struct **assocId;
    APDU *PDU_ptr;
{
    int result;

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "Calling ir_search_request ...\n");
#endif

    /* Validate the search request APDU */
    result = SUCCESS;                /* need to be resolved */

    /* service the search request */
    if (result == SUCCESS)
    {
        result = ira_service_request(assocId);
    }

    return (result);

}                                /* End Of Procedure ir_search_request */


/*****************************************************************************
 *
 * Function: ir_search_response
 *
 * Purpose: This function receives a search response from the target process,
 *          validates it, and relays the APDU to the Origin.
 *
 * Input Arguments:     assocId         - pointer to the current association
 *                      PDU_ptr         - pointer to the PDU
 *
 * External Variables:
 *
 * Return values:   SUCCESS
 *                  FAILED
 *
 * Output Argument:
 *
 * External Variables:
 *
 * Function called:    ira_send_response()
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * References:
 *
 * Notes:
 *
 ****************************************************************************/

int
ir_search_response(assocId, PDU_ptr)
    struct assoc_struct **assocId;
    APDU *PDU_ptr;
{

    int result;

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "Calling ir_search_response ...\n");
#endif

    /* is the search response APDU valid ? */
    result = SUCCESS;                /* need to be resolved */

    /* send the search response to the Origin */
    if (result == SUCCESS)
    {
        result = ira_send_response(assocId);
    }

    return (result);

}                                /* End Of Procedure ir_search_response */
