/****************************************************************************
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *~                             IR Kernel                                    ~
 *~                         Copyright(C) 1993                                ~
 *~                       Software Kinetics Ltd.                             ~
 *~                        All Rights Reserved                               ~
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright Software Kinetics Ltd.
 * 
 * Software Kinetics Ltd. has granted a licence of this software to
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve,
 * reproduce and sell the software.
 *
 * The software is provided to the user "as is" without warranty of
 * any kind.  Neither Software Kinetics Ltd. nor the National Library
 * of Canada warrants,guarantees or makes any representation regarding
 * the use, or the results of the use of the software in terms of
 * accuracy,reliability, or otherwise.  The entire risk as results and
 * performance of the software is assumed by the user. Neither
 * Software Kinetics Ltd. nor the National Library of Canada shall be
 * liable for any direct, indirect, consequential or incidental
 * damages arising out the use of software.
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 *
 * Program: oid_tool.c
 *
 * Purpose: This file contains functions for manipulating the AnsOid structures.
 *
 * Invocation:
 *
 * Input Arguments:
 *
 * Output/Exit Values: None
 *
 * Functions used:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: IR KERNEL - TARGET PRELIMINARY SOFTWARE DESIGN DOCUMENT
 *            IR KERNEL - TARGET SOFTWARE DETAIL DESIGN DOCUMENT
 *            IR KERNEL - INTERFACE REQUIREMENTS SPECIFICATION
 *            OSI - ANSI Z39.50 DOCUMENTATION
 *
 * Notes:
 *
 * $Id: oid_tool.c,v 1.2 1993/11/03 15:41:25 irsuprt Exp irsuprt $
 * $Log: oid_tool.c,v $
 * Revision 1.2  1993/11/03  15:41:25  irsuprt
 * Added the line 123 curOid->next = NULL; (PROBLEM 9.1)
 *
 * Revision 1.1  1993/11/01  17:03:51  irsuprt
 * Initial revision
 *
 *
 *****************************************************************************/

RescIR_Free(FILE * f, char *str)
{
    free(str);
}

char *
RescIR_Zmalloc(FILE * f, int size)
{
    return ((char *) malloc(size));
}

ASN_OctsCreate(char *tmpBuf, AsnLen oidByteLen, AsnOid * oidEntry)
{
    oidEntry->octetLen = oidByteLen;
    oidEntry->octs = (char *) malloc(oidByteLen);

    return (0);
}

void 
ASN_MakeOid(char *dottedString, AsnOid * oidEntry)
{
    OID *oidList;
    OID *curOid;
    OID *lastOid;
    char done = FALSE;
    long arcNum;
    char *s;
    char tmpBuf[50];
    short i;
    AsnLen oidByteLen;
    short toolStat;

    oidList = (OID *) NULL;
    curOid = oidList;
    s = dottedString;
    while (done == FALSE)
    {

        /* Skip the dot if currently positioned over one */
        while (*s == '.')
        {
            s++;
        }
        /* Extract an entry from the dotted string notation */
        i = 0;
        while ((*s != '.') && (*s != '\0'))
        {
            tmpBuf[i++] = *s++;
        }
        /* Check for last entry */
        if ((*s == '\0') && (i == 0))
        {
            done = TRUE;
        }
        else
        {
            /* Terminate the arc num string */
            tmpBuf[i] = '\0';
            /* Convert the arc num string to a long */
            arcNum = atol(tmpBuf);

            /* Malloc the OID entry. Clearing the mem. */
            curOid = (OID *) RescIR_Zmalloc(DEBUG_FILE4, sizeof(OID));
            curOid->next = NULL;

            /* Set the arc num into the OID */
            curOid->arcNum = (long int) arcNum;

            /* Create linked list of OIDs */
            if (oidList == (OID *) NULL)
            {
                /* Save the start of the list */
                oidList = curOid;
                lastOid = curOid;
            }
            else
            {
                /* Link the OID into the list of OIDs */
                lastOid->next = curOid;
                lastOid = lastOid->next;
            }
        }
    }

    oidByteLen = EncodedOidLen(oidList);

    /* Create the oid but do not populate it with oid data */
    toolStat = ASN_OctsCreate(tmpBuf, (unsigned long) oidByteLen, oidEntry);
    if (toolStat != 0)
    {
        sprintf(ERR_STR, "Error (%d) creating OidEntry", toolStat);
        display_error("ASN", DISPLAY_USER_ERROR, -1);
    }
    /* Now populate it with oid data */
    BuildEncodedOid(oidList, oidEntry);
    ASN_FreeOid(oidList);
}

ASN_FreeOid(OID * oidList)
{
    OID *tmpOid;
    OID *lastOid;

    for (tmpOid = oidList; tmpOid->next != (OID *) NULL;
            tmpOid = lastOid)
    {
        lastOid = tmpOid->next;
        RescIR_Free(DEBUG_FILE4, (char *) tmpOid);
    }
    /* Free the last one in the list */
    RescIR_Free(DEBUG_FILE4, (char *) tmpOid);
}
