/****************************************************************************
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *~                             IR Kernel                                    ~
 *~                         Copyright(C) 1993                                ~
 *~                       Software Kinetics Ltd.                             ~
 *~                        All Rights Reserved                               ~
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright Software Kinetics Ltd.
 * 
 * Software Kinetics Ltd. has granted a licence of this software to
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve,
 * reproduce and sell the software.
 *
 * The software is provided to the user "as is" without warranty of
 * any kind.  Neither Software Kinetics Ltd. nor the National Library
 * of Canada warrants,guarantees or makes any representation regarding
 * the use, or the results of the use of the software in terms of
 * accuracy,reliability, or otherwise.  The entire risk as results and
 * performance of the software is assumed by the user. Neither
 * Software Kinetics Ltd. nor the National Library of Canada shall be
 * liable for any direct, indirect, consequential or incidental
 * damages arising out the use of software.
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 *
 * Program: tpbbox.c
 *
 * Purpose: This is the main program for the target process (black box).
 *
 * Invocation: main <pipe file name>
 *
 * Input Arguments:
 *
 * Output/Exit Values: None
 *
 * Functions used:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: IR KERNEL - TARGET PRELIMINARY SOFTWARE DESIGN DOCUMENT
 *            IR KERNEL - TARGET SOFTWARE DETAIL DESIGN DOCUMENT
 *            IR KERNEL - INTERFACE REQUIREMENTS SPECIFICATION
 *            OSI - ANSI Z39.50 DOCUMENTATION
 *
 * Notes:
 *
 * $Id: tpbbox.c,v 1.3 1993/11/03 15:56:21 irsuprt Exp irsuprt $
 * $Log: tpbbox.c,v $
 * Revision 1.3  1993/11/03  15:56:21  irsuprt
 * added some include<>s (PROBLEM 20)
 *
 * Revision 1.2  1993/11/03  15:38:51  irsuprt
 * Initialized ERROR_FILE_NAME in function ir_read_default_parameter_file()
 *
 * Revision 1.1  1993/11/01  17:03:51  irsuprt
 * Initial revision
 *
 *
 *****************************************************************************/
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <rpc/rpc.h>
#include <fcntl.h>
#include <pwd.h>
#include <ctype.h>
#include <memory.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <sys/errno.h>
#include "tp.h"
#include "asn_incl.h"
#include "irerror.h"
#include "tpconst.h"
#include "ir_oid.h"

#define MAX_HOST_NAME_LEN   100
#define MaxQueueSize         5

/**************************** Global Ids *********************************/

long MAXMESSAGESIZE;
long MAX_ERROR_RECORDS;

FILE *DEBUG_FILE1, *DEBUG_FILE2, *DEBUG_FILE3, *DEBUG_FILE4;
char DEBUG_FILE_NAME1[35], DEBUG_FILE_NAME2[35], DEBUG_FILE_NAME3[35];

char ERROR_FILE_NAME[35];

int ACCESS_CONTROL_REQUIRED, RESOURCE_CONTROL_REQUIRED;

char OID_CLASS_ACCESS_CONTROL_ENCODED[35];

/**************************** Extern Global Ids **************************/

extern int errno;

/*****************************************************************************
 *
 * Function: tp_open_files
 *
 * Purpose: This function opens the debugging files.
 *
 * Input Arguments:
 *
 * Return values:
 *
 * Output Arguments:
 *
 * External Variables:
 *
 * Function used:
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
tp_open_files()
{

#ifdef DEBUG_LEVEL1
    printf("BBOX: Calling tp_open_files\n");
#endif

#ifdef DEBUG_LEVEL1
    if ((DEBUG_FILE1 = fopen(DEBUG_FILE_NAME1, "a")) == NULL)
        return (FAILED);
#endif

#ifdef DEBUG_LEVEL2
    if ((DEBUG_FILE2 = fopen(DEBUG_FILE_NAME2, "a")) == NULL)
        return (FAILED);
#endif

#ifdef DEBUG_LEVEL3
    if ((DEBUG_FILE3 = fopen(DEBUG_FILE_NAME3, "a")) == NULL)
        return (FAILED);
#endif

	/* open for oid_tool.c */
    if ((DEBUG_FILE4 = fopen("/dev/null", "a")) == NULL)
        return (FAILED);

    return (SUCCESS);

}                                /* End Of Procedure ir_tp_open_files */

/*****************************************************************************
 *
 * Function: tp_close_files
 *
 * Purpose: This function closes the debugging files.
 *
 * Input Arguments:
 *
 * Return values:
 *
 * Output Arguments:
 *
 * External Variables:
 *
 * Function used:
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
tp_close_files()
{
#ifdef DEBUG_LEVEL1
    fclose(DEBUG_FILE1);
#endif

#ifdef DEBUG_LEVEL2
    fclose(DEBUG_FILE2);
#endif

#ifdef DEBUG_LEVEL3
    fclose(DEBUG_FILE3);
#endif
}

/*****************************************************************************
 *
 * Function:    tpa_ALI_setup
 *
 * Purpose:     This function sets up the TCP Service port, gets and
 *              returns the TCP socket Fd.
 *
 * Input Arguments:
 *              tpPort - TCP service port to talk to IR kernel.
 *
 * Return values: SUCCESS or FAILED
 *
 * Output Arguments:
 *              socketFd - get the TCP socket Fd.
 *
 * External Variables:
 *
 * Function used:
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
int
tpa_ALI_setup(tpPort, socketFd)
    int *tpPort;
    int *socketFd;
{
    struct sockaddr_in serverAddress, clientAddress;
    struct hostent *hostInfo;
    char hostname[MAX_HOST_NAME_LEN], domainname[MAX_HOST_NAME_LEN];
    int i, length;

    /* Open a TCP Socket to communicate with a client */
    *socketFd = socket(AF_INET, SOCK_STREAM, 0);

    if (*socketFd < 0)
    {
        display_error("TP_INIT", ERR_CREATE_SOCKET, -1);
        return (FAILED);
    }

    /* Bind our local address so that a client can send to us */
    bzero((char *) &serverAddress, sizeof(serverAddress));
    serverAddress.sin_family = AF_INET;

    /* Get our hostname */
    if (gethostname(hostname, MAX_HOST_NAME_LEN) < 0)
    {
        display_error("TP_INIT", ERR_CANNOT_GET_HOST_NAME, errno);
        return (FAILED);
    }
    getdomainname(domainname, MAX_HOST_NAME_LEN);
	for (i=0; i<strlen(domainname); i++)
		domainname[i] = tolower(domainname[i]);
    sprintf(hostname, "%s.%s", hostname, domainname);
    hostInfo = gethostbyname(hostname);

    if (hostInfo != NULL)
    {
#ifdef VMS
        serverAddress.sin_addr.s_addr = INADDR_ANY;
#else
        serverAddress.sin_addr.s_addr = inet_network(inet_ntoa(*hostInfo->h_addr_list));
#endif
    }
    else
    {
        display_error("TP_INIT", ERR_CANNOT_GET_HOST_NAME, errno);
        return (FAILED);
    }
    serverAddress.sin_port = 0;

    length = sizeof(serverAddress);

    if (bind(*socketFd, (struct sockaddr *) & serverAddress, length) < 0)
    {
        display_error("TP_INIT", ERR_BIND_SOCKET, -1);
        return (FAILED);
    }

    if (getsockname(*socketFd, (struct sockaddr *) & serverAddress, &length) < 0)
        display_error("TP_INIT", ERR_GET_SOCKET_NAME, -1);

    *tpPort = serverAddress.sin_port;

    listen(*socketFd, MaxQueueSize);

    return (SUCCESS);

}                                /* end of tpa_ALI_setup  */

/*****************************************************************************
 *
 * Function: tp_wait_for_connect
 *
 * Purpose: This function calls the function accept() to actually connect to
 *          IR kernel and returns the new socket Fd.
 *
 * Input Arguments:
 *
 * Return values: SUCCESS or FAILED
 *
 * Output Arguments:
 *
 * External Variables:
 *              socketFd - the new TCP socket Fd from the accept().
 *
 * Function used:
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
tp_wait_for_connect(socketFd)
    int *socketFd;
{
    struct sockaddr_in serverAddress;
    int length, new_socketFd;

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "BBOX: Calling tp_wait_for_connect\n");
#endif

    /* Wait for a connection from a Client process */
    length = sizeof(serverAddress);
    new_socketFd = accept(*socketFd, (struct sockaddr *) & serverAddress, &length);
    if (new_socketFd < 0)
    {
        display_error("TP_INIT", ERR_ACCEPT_CONNECT, -1);
        return (FAILED);
    }
    else
    {
        *socketFd = new_socketFd;

#ifdef DEBUG_LEVEL3
        fprintf(DEBUG_FILE3, "BBOX:tp_wait_for_connect: accepted connect %d\n", *socketFd);
#endif

    }

    return (SUCCESS);
}

/*****************************************************************************
 *
 * Function: tpa_send_APDU
 *
 * Purpose: This function sends an APDU to IR kernel and returns number of
 *          bytes sent.
 *
 * Input Arguments:
 *              socketFd - socket Fd
 *              data_ptr - APDU data pointer
 *              APDU_length - APDU size
 *
 * Return values: Number of bytes sent
 *
 * Output Arguments:
 *
 * External Variables:
 *
 * Function used:
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
int
tpa_send_APDU(socketFd, data_ptr, APDU_length)
    int socketFd;
    SBuf *data_ptr;
    int APDU_length;
{

    int numSend;

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "BBOX:Calling tpa_send_APDU(), APDU_length=%d\n", APDU_length);
#endif

    numSend = send(socketFd, data_ptr->dataStart, APDU_length, 0);

#ifdef DEBUG_LEVEL3
    fprintf(DEBUG_FILE3, "BBOX:tpa_send_APDU: %d bytes sent\n", numSend);
#endif

    return (numSend);

}

/*****************************************************************************
 *
 * Function: tp_control
 *
 * Purpose: This function controls the traffic flow between target process
 *          and IR kernel.
 *
 * Input Arguments:
 *              socketFd - socket Fd
 *
 * Return values: SUCCESS or FAILED
 *
 * Output Arguments:
 *
 * External Variables:
 *
 * Function used:
 *              tpa_service_event()
 *              tp_process_init_request()
 *              tp_process_search_request()
 *              tp_process_present_request()
 *              tp_process_deleteResultSet_request()
 *              tp_process_triggerResourceControl_request()
 *              tp_process_resourceReport_request()
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
tp_control(socketFd)
    int socketFd;
{
    APDU PDU;
    char *data_ptr;
    int result;

    /* allocate space for all encode and decode APDU purpose */
    data_ptr = malloc(MAXMESSAGESIZE * 2);

    while (TRUE)
    {
        if (tpa_service_event(socketFd, &PDU) == FAILED)
        {
            result = FAILED;
            break;
        }

#ifdef DEBUG_LEVEL3
        fprintf(DEBUG_FILE3, "BBOX: PDU.choiceId=%d\n", PDU.choiceId);
#endif

        switch (PDU.choiceId)
        {
            case APDU_INITREQ:
                result = tp_process_init_request(&PDU, &data_ptr, socketFd);
                break;

            case APDU_SEARCHREQ:
                result = tp_process_search_request(&PDU, &data_ptr, socketFd);
                break;

            case APDU_PRESENTREQ:
                result = tp_process_present_request(&PDU, &data_ptr, socketFd);
                break;

            case APDU_DELETERESULTSETREQ:
                result = tp_process_deleteResultSet_request(&PDU, &data_ptr, socketFd);
                break;

            case APDU_TRIGGERRESOURCEREQ:
                result = tp_process_triggerResourceControl_request(&PDU, &data_ptr, socketFd);
                break;

            case APDU_RESOURCEREPORTREQ:
                result = tp_process_resourceReport_request(&PDU, &data_ptr, socketFd);
                break;

            default:

#ifdef DEBUG_LEVEL1
				fprintf(DEBUG_FILE1, "BBOX:tp_control:default case\n");
#endif

                result = FAILED;
                break;
        }

        if (result == FAILED)
            break;
    }

    free(data_ptr);

    return (SUCCESS);
}

/*****************************************************************************
 *
 * Function: ir_read_default_parameter_file
 *
 * Purpose: This function sets the default parameter MAXMESSAGESIZE from the
 *          default file irtarget.def (if it exists) and reset them.
 *
 * Input Arguments: None
 *
 * Return values: None
 *
 * Output Arguments:
 *
 * External Variables:
 *
 * Function used:
 *
 * Algorithm:
 *
 * Version: Date:   93/03/14
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: IR KERNEL - TARGET PRELIMINARY SOFTWARE DESIGN DOCUMENT
 *            IR KERNEL - TARGET SOFTWARE DETAIL DESIGN DOCUMENT
 *
 * Notes:
 *
 ****************************************************************************/
ir_read_default_parameter_file()
{
    FILE *infile;
    int i;
    char str[100];

    strcpy(ERROR_FILE_NAME,  "irtarget.err");
    strcpy(DEBUG_FILE_NAME1, "tpbbox.d1");
    strcpy(DEBUG_FILE_NAME2, "tpbbox.d2");
    strcpy(DEBUG_FILE_NAME3, "tpbbox.d3");
    MAXMESSAGESIZE = 52000L;
    MAX_ERROR_RECORDS = 1000;

    if ((infile = fopen("irtarget.def", "r")) != NULL)
    {
        if (fgets(str, 100, infile) == NULL)
        {
            display_error("KERNEL", ERR_INCOMPLETE_DEFAULT_FILE, -1);
            return (FAILED);
        }

        i = 0;
        while (fgets(str, 100, infile) != NULL)
        {
            i++;
            switch (i)
            {
                case 4:
                    sscanf(str, "%s", ERROR_FILE_NAME);
                    break;
                case 13:
                    sscanf(str, "%ld", &MAXMESSAGESIZE);
                    break;
                case 14:
                    sscanf(str, "%ld", &MAX_ERROR_RECORDS);
                    break;
                case 15:
                    sscanf(str, "%s", DEBUG_FILE_NAME1);
                    break;
                case 16:
                    sscanf(str, "%s", DEBUG_FILE_NAME2);
                    break;
                case 17:
                    sscanf(str, "%s", DEBUG_FILE_NAME3);
                    break;
            }
        }

        if (i != 17)
        {
            printf("%d entries have read from irtarget.def\n", i);
            display_error("BBOX", ERR_INCOMPLETE_DEFAULT_FILE, -1);
            return (FAILED);
        }

        fclose(infile);
    }

#ifdef DEBUG_LEVEL3
    printf("\n");
    printf("BBOX: ERROR FILE NAME             = %s\n", ERROR_FILE_NAME);
    printf("BBOX: MAX MESSAGE SIZE            = %ld\n", MAXMESSAGESIZE);
    printf("BBOX: MAX ERROR RECORDS           = %ld\n", MAX_ERROR_RECORDS);
    printf("BBOX: FIRST DEBUG FILE NAME       = %s\n", DEBUG_FILE_NAME1);
    printf("BBOX: SECOND DEBUG FILE NAME      = %s\n", DEBUG_FILE_NAME2);
    printf("BBOX: THIRD DEBUG FILE NAME       = %s\n", DEBUG_FILE_NAME3);
    printf("\n");
#endif

    return (SUCCESS);
}

/*****************************************************************************
 *
 * Function: init_ids
 *
 * Purpose: This function initializes the IDs.
 *
 * Input Arguments: None
 *
 * Return values: None
 *
 * Output Arguments:
 *
 * External Variables:
 *
 * Function used:
 *
 * Algorithm:
 *
 * Version: Date:   93/03/14
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: IR KERNEL - TARGET PRELIMINARY SOFTWARE DESIGN DOCUMENT
 *            IR KERNEL - TARGET SOFTWARE DETAIL DESIGN DOCUMENT
 *
 * Notes:
 *
 ****************************************************************************/
void
init_ids()
{
    AsnOid oid_str;

#ifdef DEBUG_LEVEL1
    printf("BBOX: Calling init_ids\n");
#endif

    InitNibbleMem(MAXMESSAGESIZE, MAXMESSAGESIZE);

    ASN_MakeOid((char *) OID_CLASS_ACCESS_CONTROL, &oid_str);
    memcpy(OID_CLASS_ACCESS_CONTROL_ENCODED, oid_str.octs, (int) oid_str.octetLen);
}

/*****************************************************************************
 *
 * Function: main
 *
 * Purpose: This is the main function of the target process (black box).
 *
 * Input Arguments:
 *
 * Return values:
 *
 * Output Arguments:
 *
 * External Variables:
 *
 * Function used:
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
main(argc, argv)
    int argc;
    char *argv[];
{
    FILE *writefd;
    char *pipename;
    int tpPort;
    int socketFd;

    if (argc > 1)
    {

        /* get a TCP port and socket Fd */
        if (tpa_ALI_setup(&tpPort, &socketFd) != SUCCESS)
            tpPort = 0;

        /* pass the TCP port to IR kernel to use for further communication */
        pipename = argv[1];        /* Create named pipe */
        if ((writefd = fopen(pipename, "w")) == NULL)
        {
            display_error("TP_BBOX", ERR_OPEN_WRITE_FILE, -1);
            exit(1);
        }
        else
        {
            fprintf(writefd, "%d\n", tpPort);
            fclose(writefd);
        }

        if (ir_read_default_parameter_file() == FAILED)
            exit(1);

        init_ids();

        tp_open_files();

#ifdef DEBUG_LEVEL1
        fprintf(DEBUG_FILE1, "BBOX:Running tp_bbox\n");
#endif

        /* if connecting to IR kerenl successful */
        if (tp_wait_for_connect(&socketFd) == SUCCESS)
        {
            tp_control(socketFd);

#ifdef DEBUG_LEVEL1
            printf("BBOX: tpbbox: close socketFd\n");
#endif

            close(socketFd);
        }

        tp_close_files();

    }                                /* end if */

#ifdef DEBUG_LEVEL1
    printf("BBOX: Main Program Exits\n");
#endif

    exit(0);
}
