/****************************************************************************
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *~                             IR Kernel                                    ~
 *~                         Copyright(C) 1993                                ~
 *~                       Software Kinetics Ltd.                             ~
 *~                        All Rights Reserved                               ~
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright Software Kinetics Ltd.
 * 
 * Software Kinetics Ltd. has granted a licence of this software to
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve,
 * reproduce and sell the software.
 *
 * The software is provided to the user "as is" without warranty of
 * any kind.  Neither Software Kinetics Ltd. nor the National Library
 * of Canada warrants,guarantees or makes any representation regarding
 * the use, or the results of the use of the software in terms of
 * accuracy,reliability, or otherwise.  The entire risk as results and
 * performance of the software is assumed by the user. Neither
 * Software Kinetics Ltd. nor the National Library of Canada shall be
 * liable for any direct, indirect, consequential or incidental
 * damages arising out the use of software.
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 *
 * Program: tpstartu.c
 *
 * Purpose: This is the init RPC function which executes the program 'tpbbox'
 *          and then reads the TCP port number via name pipe from the
 *          program 'tpbbox'. Finally, it returns this TCP port number to
 *          the IR kernel init program.
 *
 * Invocation:
 *
 * Input Arguments:
 *
 * Output/Exit Values: None
 *
 * Functions used:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: IR KERNEL - TARGET PRELIMINARY SOFTWARE DESIGN DOCUMENT
 *            IR KERNEL - TARGET SOFTWARE DETAIL DESIGN DOCUMENT
 *            IR KERNEL - INTERFACE REQUIREMENTS SPECIFICATION
 *            OSI - ANSI Z39.50 DOCUMENTATION
 *
 * Notes:
 *
 * $Id: tpstartu.c,v 1.3 1993/11/03 15:48:16 irsuprt Exp irsuprt $
 * $Log: tpstartu.c,v $
 * Revision 1.3  1993/11/03  15:48:16  irsuprt
 * Added #include <sys/stat.h> (PROBLEM 20)
 *
 * Revision 1.2  1993/11/02  15:42:35  irsuprt
 * changed #ifdef 0 to #ifdef ACTIVE_TEMPORARY of line 152,172 for ANSI
 * compliance (PROBLEM 13)
 *
 * Revision 1.1  1993/11/01  17:03:51  irsuprt
 * Initial revision
 *
 *
 *****************************************************************************/
#include <stdio.h>
#include <rpc/rpc.h>
#include <fcntl.h>
#include <pwd.h>
#include <ctype.h>
#include <sys/errno.h>
#include <sys/stat.h>

#define ERROR -1
#define PERMS 0666
extern int errno;
extern int TCP_PORT_NO;

#ifdef VMS
int *
tp_startup_1(void)
{
    int pid;
    int pipe_fd[2];
    FILE *readfd;

    if (pipe(pipe_fd) < 0)
    {
        puts("Could not create pipe");
        return (int *) 0;
    }

    if ((pid = vfork()) == 0)
    {
        char buf[64];

        /* Child process */
        puts("TPSTARTUP child: about to exec tp_init...");
        sprintf(buf, "%d", pipe_fd[1]);
        execl("tp_init", "tp_init", buf, NULL);
        perror("Error executing `tp_init'");
        _exit(1);
    }

    /* printf("TPSTARTUP parent: child pid was %d (0x%X)\n", pid, pid); */

    if ((readfd = fdopen(pipe_fd[0], "r")) == NULL)
    {
        perror("Error doing fdopen() on pipe");
        TCP_PORT_NO = 0;
    }
    else
        fscanf(readfd, "%d", &TCP_PORT_NO);

    close(pipe_fd[0]);
    close(pipe_fd[1]);

    /* printf("TPSTARTUP parent: tp_startup_1: TCP_PORT_NO=%d\n", TCP_PORT_NO); */

    return &TCP_PORT_NO;
}

#else

int *
tp_startup_1()
{

    char pipename[16];
    int tcp_port, pid;
    FILE *readfd;
    int wpid, status;

    sprintf(pipename, "/tmp/pipef%05d", getpid());

    /* printf("Pipe Name: %s\n", pipename); */

    if ((mknod(pipename, S_IFIFO | PERMS | O_NDELAY, 0) < 0) && (errno != EEXIST))
    {
        printf("Could not create FIFO.\n");
    }
    else
    {
        pid = fork();
        if (pid == 0)
        {                        /* child process */
            /* printf("execl tpbbox\n"); */
            execl("tpbbox", "tpbbox", pipename, 0);        /* start target process */
        }

        while ((readfd = fopen(pipename, "r")) == NULL)
            printf("Could not open FIFO for reading.\n");
        fscanf(readfd, "%d\n", &tcp_port);
        TCP_PORT_NO = tcp_port;
        fclose(readfd);
        unlink(pipename);

#ifdef ACTIVE_TEMPORARY
        /* parent checks for a child has finished */
        while ((wpid = wait(&status)) != 0)
        {
            printf("child process has terminated (wpid=%d)\n", wpid);
        }
        else
        if (wpid == ERROR && errno != EINTR)
        {
            printf("wait/return error\n");
        }
#endif

    }
    /* printf("tp_startup_1: TCP_PORT_NO=%d\n", TCP_PORT_NO); */

    return (&TCP_PORT_NO);
}
#endif

#ifdef ACTIVE_TEMPORARY
tp_init(argc, argv)
    int argc;
    char *argv[];
{

    char pipename[15];
    int tcp_port, pid;
    FILE *readfd;

    strcpy(pipename, "/tmp/piptest");
    /* printf("Pipe Name: %s\n", pipename); */
    if ((mknod(pipename, S_IFIFO | PERMS | O_NDELAY, 0) < 0) && (errno != EEXIST))
    {
        printf("Could not create FIFO.\n");
    }
    else
    {
        pid = fork();
        if (pid == 0)
        {                        /* child process */
            /* printf("execl tp_init\n"); */
            execl("tp_init", "tp_init", pipename, 0);        /* start qrmhandler */
        }                        /* end if child */

        while ((readfd = fopen(pipename, "r")) == NULL)
            printf("Could not open FIFO for reading.\n");
        fscanf(readfd, "%d\n", &tcp_port);
        /* printf("tp_init: TCP Port: %d\n", tcp_port); */
        fclose(readfd);
        unlink(pipename);
    }
}
#endif
